"""Base client module for Sleep interactions."""

import logging
from .config import SleepConfig

# Configure logging
logger = logging.getLogger("mcp-sleep")


class SleepClient:
    """Base client for Sleep interactions."""

    def __init__(self, config: SleepConfig | None = None) -> None:
        """Initialize the Sleep client with given or environment config.

        Args:
            config: Configuration for Sleep client. If None, will load from
                environment.

        Raises:
            ValueError: If configuration is invalid or environment variables
            are missing
        """
        self.config = config or SleepConfig.from_env()



"""Configuration module for the Sleep."""

import os
from dataclasses import dataclass


@dataclass
class SleepConfig:
    """Sleep configuration."""
    timeout: int  # Max wait timeout

    @classmethod
    def from_env(cls) -> "SleepConfig":
        """Create configuration from environment variables.

        Returns:
            SleepConfig with values from environment variables

        Raises:
            ValueError: If any required environment variable is missing
        """
        timeout = int(os.getenv("MCP_SLEEP_TIMEOUT"))

        return cls(
            timeout=timeout
        )



import logging
from collections.abc import AsyncIterator
from contextlib import asynccontextmanager
from dataclasses import dataclass
from typing import Any, Sequence
from mcp import Tool
from mcp.server import Server
from mcp.types import TextContent

from .sleep import SleepFetcher


# Configure logging
logger = logging.getLogger("mcp-sleep")


@dataclass
class AppContext:
    """Application context for MCP Sleep."""
    sleep: SleepFetcher | None = None


@asynccontextmanager
async def server_lifespan(server: Server) -> AsyncIterator[AppContext]:
    """Initialize and clean up application resources."""
    try:
        # Initialize services
        sleep = SleepFetcher()

        # Log the startup information
        logger.info("Starting MCP Sleep server")

        # Provide context to the application
        yield AppContext(sleep=sleep)
    finally:
        # Cleanup resources if needed
        pass


# Create server instance
app = Server("mcp-sleep", lifespan=server_lifespan)


@app.list_tools()
async def list_tools() -> list[Tool]:
    """List available Sleep tools."""
    tools = []
    ctx = app.request_context.lifespan_context

    # Add Sleep tools if Sleep is configured
    if ctx:
        tools.extend(
            [
                Tool(
                    name="sleep",
                    description="If you need to wait for a few seconds to continue with the task you are performing .",  # noqa: E501
                    inputSchema={
                        "type": "object",
                        "properties": {
                            "seconds": {
                                "type": "number",
                                "minimum": 0,
                                "maximum": ctx.sleep.config.timeout,
                                "description": "seconds it will take me to tell you to continue."  # noqa: E501
                            }
                        },
                        "required": ["seconds"],
                    },
                ),
            ]
        )
    return tools


@app.call_tool()
async def call_tool(name: str, arguments: Any) -> Sequence[TextContent]:
    """Handle tool calls for Sleep."""
    ctx = app.request_context.lifespan_context
    try:
        if name == "sleep":
            if not ctx:
                raise ValueError("Sleep is not configured.")
            seconds = arguments.get("seconds")
            if not seconds:
                raise ValueError("Seconds is required.")

            if seconds > int(ctx.sleep.config.timeout):
                return [
                    TextContent(
                        type="text",
                        text=f"My configuration does not allow me to make you wait more than {ctx.sleep.config.timeout} seconds",  # noqa: E501
                    )
                ]
            # Sleep seconds value
            await ctx.sleep.wait(seconds)

            return [
                TextContent(
                    type="text",
                    text=f"You can continue to do your task after you've been waiting {seconds} seconds",  # noqa: E501
                )
            ]

        raise ValueError(f"Unknown tool: {name}")

    except Exception as e:
        logger.error(f"Tool execution error: {str(e)}")
        return [TextContent(type="text", text=f"Error: {str(e)}")]


async def run_server(transport: str = "stdio", port: int = 8000) -> None:
    """Run the MCP Sleep server with the specified transport."""
    if transport == "sse":
        from mcp.server.sse import SseServerTransport
        from starlette.applications import Starlette
        from starlette.requests import Request
        from starlette.routing import Mount, Route

        sse = SseServerTransport("/messages/")

        async def handle_sse(request: Request) -> None:
            async with sse.connect_sse(
                request.scope, request.receive, request._send
            ) as streams:
                await app.run(
                    streams[0], streams[1], app.create_initialization_options()
                )

        starlette_app = Starlette(
            debug=True,
            routes=[
                Route("/sse", endpoint=handle_sse),
                Mount("/messages/", app=sse.handle_post_message),
            ],
        )

        import uvicorn

        # Set up uvicorn config
        config = uvicorn.Config(starlette_app, host="0.0.0.0", port=port)  # noqa: S104,E501
        server = uvicorn.Server(config)
        # Use server.serve() instead of run() to stay in the same event loop
        await server.serve()
    else:
        from mcp.server.stdio import stdio_server

        async with stdio_server() as (read_stream, write_stream):
            await app.run(
                read_stream, write_stream, app.create_initialization_options()
            )



# flake8: noqa: E501
"""Sleep integration module.

This module provides access to Sleep through the Model Context
Protocol.
"""

from .wait import WaitMixin
from .client import SleepClient
from .config import SleepConfig

class SleepFetcher(WaitMixin):
    """Main entry point for Sleep operations, providing backward
    compatibility.

    This class combines functionality from various mixins to maintain the same
    API as the original SleepFetcher class.
    """

    pass

__all__ = ["SleepFetcher","SleepConfig", "SleepClient"]



import asyncio
import logging
import os
import sys

import click
from dotenv import load_dotenv

__version__ = "0.1.1"

logger = logging.getLogger("mcp-sleep")


@click.command()
@click.option(
    "-v",
    "--verbose",
    count=True,
    help="Increase verbosity (can be used multiple times)",
)
@click.option(
    "--env-file", type=click.Path(exists=True, dir_okay=False),
    help="Path to .env file"
)
@click.option(
    "--timeout",
    help="Maximun time in seconds that the MCP server will wait for",
)
@click.option(
    "--transport",
    type=click.Choice(["stdio", "sse"]),
    default="stdio",
    help="Transport type (stdio or sse)",
)
@click.option(
    "--port",
    default=8000,
    help="Port to listen on for SSE transport",
)
def main(
    verbose: bool,
    env_file: str | None,
    transport: str,
    timeout: int,
    port: int,
) -> None:
    """MCP Sleep Server

    """
    # Configure logging based on verbosity
    logging_level = logging.INFO
    if verbose == 1:
        logging_level = logging.INFO
    elif verbose >= 2:
        logging_level = logging.DEBUG

    logging.basicConfig(level=logging_level, stream=sys.stderr)

    # Load environment variables from file if specified, otherwise try default .env   # noqa: E501
    if env_file:
        logger.debug(f"Loading environment from file: {env_file}")
        load_dotenv(env_file)
    else:
        logger.debug("Attempting to load environment from default .env file")
        load_dotenv()

    # Set environment variables from command line arguments if provided
    if timeout:
        os.environ["MCP_SLEEP_TIMEOUT"] = timeout
    elif os.environ.get('MCP_SLEEP_TIMEOUT') is None:
        os.environ["MCP_SLEEP_TIMEOUT"] = str(60)  # Default value

    from . import server as server

    # Run the server with specified transport
    asyncio.run(server.run_server(transport=transport, port=port))


__all__ = ["main", "server", "__version__"]



"""Module for Sleep waits operations."""

import asyncio
import logging

from .client import SleepClient

logger = logging.getLogger("mcp-sleep")


class WaitMixin(SleepClient):
    """Mixin for Sleep waits operations."""

    async def wait(self, seconds: int) -> None:
        """
        Get an aggregated overview of findings and resources grouped by providers.

        Returns:
            Dictionary containing provider information with results and
            metadata
        """  # noqa: E501
        await asyncio.sleep(seconds)
        return
